using System.Collections.Generic;
using System.Data.Common;

namespace AncouApp.ChoBase.Core
{
	
    internal static class KvsAccesser
    {
        #region Sqlite infomation
        internal const string TableName = "ChoBase";
        internal const string ColumnKey = "KvsKey";
        internal const string ColumnValue = "KvsValue";
        internal const string ColumnUpdated = "Updated";
        internal const string OrderParamname = "OrderDirection";

        private static DbConnection _connection;
        #endregion

        #region SQL
        private const string QueryFindByID = "select KvsValue from ChoBase where KvsKey = :KvsKey";
        private const string QueryScanByID = "select KvsValue from ChoBase where KvsKey like :KvsKey order by KvsKey {0}";
        private const string QueryCountByKey = "select count(KvsValue) from ChoBase where KvsKey like :KvsKey";
        private const string QueryDeleteByKey = "delete from ChoBase where KvsKey = :KvsKey";
        private const string QueryReplace = "replace into ChoBase (KvsKey,KvsValue,Updated) values (:KvsKey,:KvsValue,current_timestamp)";
        private const string QuerySelectTableName = "SELECT count(*) FROM SQLITE_MASTER WHERE TYPE ='table' AND NAME = '{0}'";

        #endregion

        #region Access Method

        /// <summary>
        /// select value with key.
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static string SelectByKey(string key)
        {
            using (var cmd = Connection.CreateCommand())
            {
                cmd.CommandText = QueryFindByID;
                KvsAccesser.AddParameter(cmd, ColumnKey, key);
                DbDataReader reader = cmd.ExecuteReader();
                if (reader.Read())
                {
                    return (string)reader[ColumnValue];
                }
            }
            throw new KeyNotFoundException(key.ToString());
        }

        /// <summary>
        ///Select TabIndex like 'arg%' 
        /// </summary>
        /// <param name="key">
        /// A <see cref="System.String"/>
        /// </param>
        /// <param name="direction">
        /// A <see cref="SortDirections"/>
        /// </param>
        /// <returns>
        /// A <see cref="IList<System.String>"/>
        /// </returns>
        public static IList<string> ScanByKey(string key, SortDirections direction)
        {
            using (var cmd = Connection.CreateCommand())
            {
                IList<string> rtn = new List<string>();
                cmd.CommandText = string.Format(QueryScanByID , direction.ToString());
                KvsAccesser.AddParameter(cmd, ColumnKey, key + "%");
                DbDataReader reader = cmd.ExecuteReader();
                while (reader.Read())
                {
                    rtn.Add((string)reader[ColumnValue]);
                }
                return rtn;
            }
        }
		
        public static int DeleteByKey(string key)
        {
            using (var cmd = Connection.CreateCommand())
            {
                cmd.CommandText = QueryDeleteByKey;
                KvsAccesser.AddParameter(cmd, ColumnKey, key);
				int rtn = cmd.ExecuteNonQuery();
				return rtn ;
            }
			
        }
		
        public static void Merge(string key, string val)
        {
            using (var cmd = Connection.CreateCommand())
            {
                cmd.CommandText = QueryReplace;
                KvsAccesser.AddParameter(cmd, ColumnKey, key);
                KvsAccesser.AddParameter(cmd, ColumnValue, val);
                cmd.ExecuteNonQuery();
            }
        }

        #endregion

        /// <summary>
        /// Add new parameter to dbCommand.
        /// Create DbParameter by columnName and value;
        /// </summary>
        /// <param name="cmd"></param>
        /// <param name="columnName"></param>
        /// <param name="value"></param>
        internal static DbParameter AddParameter(DbCommand cmd, string columnName, object value)
        {
            DbParameter parameter = cmd.CreateParameter();
            parameter.ParameterName = columnName;
            parameter.Value = value;
            cmd.Parameters.Add(parameter);
            return parameter;
        }


        #region Connection

        /// <summary>
        /// Get Connection
        /// </summary>
        public static DbConnection Connection
        {
            get
            {
                if (_connection == null)
                {
                    bool fileExist = ConnectionUtil.ExistDbFile;
                    _connection = ConnectionUtil.GetNewConnection();
                    if (!fileExist) { CreateTable(); }
                }
                return _connection;
            }
        }


        /// <summary>
        /// Create New Table 
        /// </summary>
        /// <param name="con">
        /// A <see cref="SqliteConnection"/>
        /// </param>
        private static void CreateTable()
        {
            using (DbCommand cmd = _connection.CreateCommand())
            {
                string createTableSql = "	create table " + TableName + "(	" +
                   "	" + ColumnKey + " text  not null ,	" +
                   "	" + ColumnValue + " text not null ,	" +
                   "	" + ColumnUpdated + " numeric  not null ,	" +
                   "	primary key (" + ColumnKey + ")" +
                   "	)	";
                cmd.CommandText = createTableSql;
                cmd.ExecuteNonQuery();
            }
        }
        #endregion
    }
}
